package jdeprscan2;

import java.sql.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.derby.iapi.reference.ClassName;

public class DerbyUtil {

    public static final String embdriver = "EmbeddedDriver";
    public static final String clidriver = "ClientDriver";
    public static final String lhost = "localhost";
    public static final int port = 1527;
    public static final String separator = System.getProperty("file.separator");
    public static final String createTable1 = "CREATE TABLE klasy(plik "
            + "VARCHAR(256), klasa BLOB)";
    public static final String createTableSource = "CREATE TABLE klasy(plik "
            + "VARCHAR(256), klasa CLOB)";

    private DerbyUtil() {
    }

    //---------------------- R007 ------------------------------------------
    /**
     * Startuje DerbyEngine. Jeśi załadowano Embedded Driver można od razu
     * połączyć się z bazą danych. Jeśli załadowano ClientDriver aby połączyć
     * się z bazą danych należy wcześniej uruchomić NetworkServer na tym lub
     * zdalnym komputerze (w zależności gdzie znajduje się baza danych). Jeśli
     * baza danych znajduje się na innym komputerze to połączenie z nią jest
     * możliwe o ile na komputerze z bazą zostaną ustawione odpowiednie prawa
     * dostępu. Jeżeli nie zostaną ustawione możliwe jest połączenie tylko z
     * tego komputera, na którym baza się znajduje
     *
     * @param driverName - nazwa sterownika
     * @return - true -jeśli sterownik został załadowany i silnik bazy danych
     * Derby został uruchomiony, false - jeśli sterownik bazy danych nie został
     * załadowany i silnik bazy danych nie został uruchomiony
     */
    public static boolean startDerbyEngine(String driverName) {
        return loadDBDriver(driverName);
    }

    /**
     * Ładuje wskazany sterownik bazy danych
     *
     * @param driverName - nazwa sterownika do załadowania
     * @return - true - jeśli sterownik został załadowany, false - w przeciwnym
     * przypadku
     * @throws IllegalArgumentException
     */
    private static boolean loadDBDriver(String driverName)
            throws IllegalArgumentException {
        if (driverName.equals("EmbeddedDriver")
                || driverName.equals("ClientDriver")) {
            try {
                try {
                    Class.forName("org.apache.derby.jdbc." + driverName)
                            .newInstance();
                } catch (InstantiationException | IllegalAccessException e) {
                    Logger.getLogger(ClassName.class.getName()).log(Level.SEVERE, null, e);
                }
            } catch (ClassNotFoundException e) {
                Logger.getLogger(ClassName.class.getName()).log(Level.SEVERE, null, e);
                return false;
            }
        } else {
            throw new IllegalArgumentException("Niewłaściwa nazwa sterownika");
        }
        return true;
    }

    /**
     * Tworzy połaczenie z wbudowaną bazą danych. Ustawia właściwości określone
     * w łańcuchu znaków 'properties'
     *
     * @param dbName - nazwa bazy danych lub ścieżka do bazy danych z nazwą bazy
     * danych na końcu
     * @param properties - pary właściwości połączenia (nazwa=właściwość)
     * oddzielone średnikami. Średnik na początku i końcu nie są potrzebne.
     * @return - połączenie z bazą danych
     */
    public static Connection connectEmbeddedDB(String dbName,
            String properties) {
        Connection conn = null;
        try {
            conn = DriverManager
                    .getConnection("jdbc:derby:" + dbName + ";" + properties);
        } catch (SQLException ex) {
            Logger.getLogger(ClassName.class.getName()).log(Level.SEVERE, null, ex);
        }
        return conn;
    }

    /**
     * Zamyka polecenie
     *
     * @param stat - polecenie do zamknięcia
     */
    public static boolean close(Statement stat) {
        if (stat != null) {
            try {
                stat.close();
            } catch (SQLException e) {
                Logger.getLogger(ClassName.class.getName()).log(Level.SEVERE, null, e);
            }
        }
        return true;
    }

    /**
     * Zamyka połączenie
     *
     * @param conn - połączenie do zamknięcia
     */
    public static boolean close(Connection conn) {
        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException e) {
                Logger.getLogger(ClassName.class.getName()).log(Level.SEVERE, null, e);
            }
        }
        return true;
    }

    /**
     * Zamyka bazę o podanej nazwie
     *
     * @param dbName - nazwa bazy do zamknięcia
     * @return true - jesli baza zostala zamknieta, false - w przeciwnym wypadku
     */
    public static boolean shutdownEmbeddedDB(String dbName) {
        boolean shutted = false;
        boolean gotSQLExc = false;
        try {
            DriverManager
                    .getConnection("jdbc:derby:" + dbName + ";shutdown=true");
        } catch (SQLException se) {
            if (se.getSQLState().equals("XJ015")) {
                gotSQLExc = true;
            }
        }
        shutted = gotSQLExc;
        return shutted;
    }

    /**
     * Zatrzymuje silnik bazy danych Derby i wszystkie bazy danych,
     *
     * @return - true - jeśli silnik został zatrzymany i wszystkie bazy zostały
     * zamknięte, false - w przeciwnym przypadku.
     */
    public static boolean shutdownDerbyEngine() {
        boolean gotSQLExc = false;
        try {
            DriverManager.getConnection("jdbc:derby:;shutdown=true");
        } catch (SQLException se) {
            if (se.getSQLState().equals("XJ015")) {
                gotSQLExc = true;
            }
        }
        return gotSQLExc;
    }

    /**
     * Zamyka zbiór wynikowy
     *
     * @param rs - zamykany zbiór wynikowy
     */
    public static boolean close(ResultSet rs) {
        if (rs != null) {
            try {
                rs.close();
            } catch (SQLException e) {
                Logger.getLogger(ClassName.class.getName()).log(Level.SEVERE, null, e);
            }
        }
        return true;
    }

}
